import setuptools

setuptools.setup(
	name="gandan", # Replace with your own username
	version="0.0.0",
	author="SANG-HUN, KIM",
	author_email="fury8208@gmail.com",
	description="a small tcp based middlware",
	url="https://github.com/willcaster0418/gandan",
	packages=["gandan"],
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	]
)
